@@ -5,7 +5,7 @@
 void fhm(double *v, int *n);
 void out(double *d, double *r, int *n, int *k, int *l);
 void fitch6(int *dat, int *nr, int *pars, int *node, int *edge, int *nl, double *weight, double *pvec, double *pscore); 
-void FN(int *dat, int *res, int *nr, int *pars, int *node, int *edge, int *nl, int *pc, double *weight);
+void FN2(int *dat, int *res, int *nr, int *pars, int *node, int *edge, int *nl, int *pc, double *weight, double *tmpvec, double *pvec);
 void giveIndex(int *left, int* right, int *ll, int *lr, int *n, int *res);
 void sankoff4(double *dat, int n, double *cost, int k, double *result);
 void reorder(int *from, int *to, int *n, int *sumNode,  int *neworder, int *root);
